package com.gree.mall.contest.config.wx;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.api.impl.WxMaServiceImpl;
import cn.binarywang.wx.miniapp.config.impl.WxMaDefaultConfigImpl;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.service.WxPayService;
import com.github.binarywang.wxpay.service.impl.WxPayServiceImpl;
import com.google.common.collect.Maps;
import com.gree.mall.contest.logic.wxmp.WxMpMessageLogic;
import com.gree.mall.contest.plus.entity.AdminCompanyWechat;
import com.gree.mall.contest.plus.service.AdminCompanyWechatService;
import com.gree.mall.contest.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.mp.api.WxMpMessageHandler;
import me.chanjar.weixin.mp.api.WxMpMessageRouter;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;


import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

@Component
@Configuration
@Slf4j
public class WxConfiguration {

    private final static String keyPath = "classpath:/static/apiclient_cert.p12";
    @Value("${wechat.payment.notifyUrl}")
    private String notifyUrl;

    //小程序服务
    public static Map<String, WxMaService> wxMaService = Maps.newConcurrentMap();
    //微信公众号
    public static Map<String, WxMpService> wxMpServices = Maps.newConcurrentMap();
    //微信公众号推送消息
    public static Map<String, WxMpMessageRouter> wxMpMessageRouters = Maps.newConcurrentMap();
    //微信支付服务 key = companyWechatId
    public static Map<String, WxPayService> wxPayServices = Maps.newConcurrentMap();
    //发货模板  key = companyWechatId
    public static Map<String, String> temFhNotice = Maps.newConcurrentMap();

    @Autowired
    RedisUtil redisUtil;
    @Autowired
    AdminCompanyWechatService AdminCompanyWechatService;


    @PostConstruct
    public void initWxServices() {
        //查询企业微信配置
        List<AdminCompanyWechat> list = AdminCompanyWechatService.list();
        reloadWxConfig(list);

    }

    public void reloadWxConfig(List<AdminCompanyWechat> list) {

        for (AdminCompanyWechat companyWechat : list) {
            if (StringUtils.isBlank(companyWechat.getSubAppId()) || StringUtils.isBlank(companyWechat.getSubSecret())) {
                continue;
            }
            //初始化微信小程序服务
            initWxMaService(companyWechat, redisUtil);
            //初始化微信公众号服务
            initWxMpService(companyWechat, redisUtil);
            //初始化微信支付服务
            initWxPayService(companyWechat, keyPath, notifyUrl);
        }
    }

    /**
     * 初始化小程序服务
     */
    private static void initWxMaService(AdminCompanyWechat companyWechat, RedisUtil redisUtil) {
        if (StringUtils.isAnyBlank(companyWechat.getSubAppId(), companyWechat.getSubSecret())) {
            return;
        }
        WxMaDefaultConfigImpl config = new WxMaLettuceRedisConfig(redisUtil);
        // 使用上面的配置时，需要同时引入jedis-lock的依赖，否则会报类无法找到的异常
        config.setAppid(companyWechat.getSubAppId());
        config.setSecret(companyWechat.getSubSecret());
        WxMaService service = new WxMaServiceImpl();
        service.setWxMaConfig(config);
        wxMaService.put(companyWechat.getSubAppId(), service);
        temFhNotice.put(companyWechat.getCompanyWechatId(), companyWechat.getTemplate());
    }

    /**
     * 初始化微信公众号服务
     *
     * @param companyWechat
     * @param redisUtil
     */
    private static void initWxMpService(AdminCompanyWechat companyWechat, RedisUtil redisUtil) {
        if (StringUtils.isBlank(companyWechat.getPubAppId()) || StringUtils.isBlank(companyWechat.getPubAppSecret())) {
            return;
        }
        log.info("开始初始化微信公众号服务: {}", companyWechat);
        WxMpService service = new WxMpServiceImpl();
        WxMpDefaultConfigImpl wxMpDefaultConfig = new WxMpLettuceRedisConfig(redisUtil);
        wxMpDefaultConfig.setAppId(companyWechat.getPubAppId());
        wxMpDefaultConfig.setSecret(companyWechat.getPubAppSecret());
        wxMpDefaultConfig.setToken("zfiresxb");
        wxMpDefaultConfig.setAesKey("2lJ7n6OOZG4xJhCtgPSmM4b8vmOffhIhpwkEJntX151");
        service.setWxMpConfigStorage(wxMpDefaultConfig);

        // 推送消息处理
        WxMpMessageHandler textHandler = new WxMpMessageLogic();

        WxMpMessageRouter wxMpMessageRouter = new WxMpMessageRouter(service);
        wxMpMessageRouter.rule().handler(textHandler).end();
        wxMpServices.put(companyWechat.getCompanyWechatId(), service);
        wxMpMessageRouters.put(companyWechat.getCompanyWechatId(), wxMpMessageRouter);
        log.info("结束初始化微信公众号服务");
    }

    /**
     * 初始化微信支付服务
     *
     * @param companyWechat
     */
    private static void initWxPayService(AdminCompanyWechat companyWechat, String keyPath, String notifyUrl) {

        String path = keyPath.toLowerCase();
        WxPayConfig wxPayConfig = new WxPayConfig();
        wxPayConfig.setAppId(companyWechat.getPubAppId());
        wxPayConfig.setMchId(companyWechat.getMchId());
        wxPayConfig.setMchKey(companyWechat.getMchKey());
        wxPayConfig.setSubAppId(companyWechat.getSubAppId());
        wxPayConfig.setSubMchId(companyWechat.getSubMchId());
        wxPayConfig.setNotifyUrl(notifyUrl);
        wxPayConfig.setKeyPath(path);
        WxPayService wxPayService = new WxPayServiceImpl();
        wxPayService.setConfig(wxPayConfig);
        wxPayServices.put(companyWechat.getCompanyWechatId(), wxPayService);
    }

}
