package com.gree.mall.contest.controller.pc.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.bean.goods.*;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.goods.GoodsLogic;
import com.gree.mall.contest.plus.entity.User;
import com.gree.mall.contest.utils.excel.ExcelUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "商品列表", description = "商品列表")
@RequestMapping(value = "/goods", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class GoodsController {

    private final GoodsLogic goodsLogic;

    @GetMapping("/count")
    @Operation(summary = "分类统计")
    public ResponseHelper<GoodsTypeCount> count(HttpServletRequest request){
        GoodsTypeCount goodsTypeCount = goodsLogic.countByType(request);
        return ResponseHelper.success(goodsTypeCount);
    }

    @GetMapping("/list/page")
    @Operation(summary = "商品列表")
    public ResponseHelper<IPage<GoodsSpecBean>> page(
            @Parameter(description = "商品标志 S=出售中 O=售罄 W=仓库中 A=活动中") @RequestParam(required = false) String flag,
            @Parameter(description = "商品关键字") @RequestParam(required = false) String keyword,
            @Parameter(description = "销售价格开始区间") @RequestParam(required = false) BigDecimal startPrice,
            @Parameter(description = "销售价格结束区间") @RequestParam(required = false) BigDecimal endPrice,
            @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal startShare,
            @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal endShare,
            @Parameter(description = "排序json格式[{sortColumn: 'PRICE', type: true },...] " +
                    "sortColumn:商品排序对应值 PRICE=销售价格 SHARE=分销金额 SOLD=销量 STOCK=库存, " +
                    "type: true=升序 false=倒序")
            @RequestParam(required = false) String sortJson,
            @Parameter(description = "商品分类 传入商品小类") @RequestParam(required = false) String categoryId,
            @Parameter(description = "商品状态 true:上架  false:下架") @RequestParam(required = false) Boolean status,
            @Parameter(description = "商品类型：COMMON=普通商品 PACKAGE=套购商品,不传默认为普通商品,支持传多个",required = false)
            @RequestParam(required = false) List<String> goodsTypes,
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize,
            HttpServletRequest request
    ) throws Exception {
        IPage<GoodsSpecBean> goodsSpecBeanIPage = goodsLogic.page(flag, keyword, startPrice, endPrice,
                startShare, endShare, sortJson, categoryId, status,goodsTypes, pageNum, pageSize , request);
        return ResponseHelper.success(goodsSpecBeanIPage);
    }

    @GetMapping("/list/export")
    @Operation(summary = "导出excel")
    public void downloadFile (HttpServletRequest request, HttpServletResponse response,
                              @Parameter(description = "商品标志 S=出售中 O=售罄 W=仓库中 A=活动中") @RequestParam(required = false) String flag,
                              @Parameter(description = "商品关键字") @RequestParam(required = false) String keyword,
                              @Parameter(description = "销售价格开始区间") @RequestParam(required = false) BigDecimal startPrice,
                              @Parameter(description = "销售价格结束区间") @RequestParam(required = false) BigDecimal endPrice,
                              @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal startShare,
                              @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal endShare,
                              @Parameter(description = "排序json格式[{sortColumn: 'PRICE', type: true },...] " +
                                      "sortColumn:商品排序对应值 PRICE=销售价格 SHARE=分销金额 SOLD=销量 STOCK=库存, " +
                                      "type: true=升序 false=倒序")
                                  @RequestParam(required = false) String sortJson,
                              @Parameter(description = "商品分类 传入商品小类") @RequestParam(required = false) String categoryId,
                              @Parameter(description = "商品状态 true:上架  false:下架") @RequestParam(required = false) Boolean status,
                              @Parameter(description = "商品类型：COMMON=普通商品 PACKAGE=套购商品,不传默认为普通商品,支持传多个",required = false) @RequestParam(required = false) List<String> goodsTypes
    ) throws Exception {
        goodsLogic.export( request,  response, flag, keyword, startPrice, endPrice,
                startShare, endShare, sortJson, categoryId, status,goodsTypes);
    }


    @GetMapping("/package/list")
    @Operation(summary = "套餐商品-列表")
    public ResponseHelper<IPage<GoodsPackageList>> packageList(
            @Parameter(description = "商品关键字") @RequestParam(required = false) String keyword,
            @Parameter(description = "销售价格开始区间") @RequestParam(required = false) BigDecimal startPrice,
            @Parameter(description = "销售价格结束区间") @RequestParam(required = false) BigDecimal endPrice,
            @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal startShare,
            @Parameter(description = "分销金额开始区间") @RequestParam(required = false) BigDecimal endShare,
            @Parameter(description = "排序json格式[{sortColumn: 'PRICE', type: true },...] " +
                    "sortColumn:商品排序对应值 PRICE=销售价格 SHARE=分销金额 SOLD=销量 STOCK=库存, " +
                    "type: true=升序 false=倒序")
            @RequestParam(required = false) String sortJson,
            @Parameter(description = "商品分类 传入商品小类") @RequestParam(required = false) String categoryId,
            @Parameter(description = "商品状态 true:上架  false:下架") @RequestParam(required = false) Boolean status,
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize,
            HttpServletRequest request
    ) throws Exception {
        IPage<GoodsPackageList> goodsPackageListIPage = goodsLogic.packageList(keyword, startPrice, endPrice,
                startShare, endShare, sortJson, categoryId, status, pageNum, pageSize, request);
        return ResponseHelper.success(goodsPackageListIPage);
    }



    @GetMapping("/listbycategory")
    @Operation(summary = "商品详情")
    public ResponseHelper<List<GoodsSpecBean>> listbycategory(
            @Parameter(description = "商品分类 传入商品小类") @RequestParam(required = false) List<String> categoryIds,
            HttpServletRequest request
    ) throws RemoteServiceException {
        List<GoodsSpecBean> listbycategory = goodsLogic.listbycategory(categoryIds,request);
        return ResponseHelper.success(listbycategory);
    }

    @PostMapping("/batch/category/update")
    @Operation(summary = "批量修改分类")
    public ResponseHelper batchUpdate(@RequestBody GoodsCategoryUpdateBean goodsCategoryUpdateBean){
        goodsLogic.updateCategory(goodsCategoryUpdateBean);
        return ResponseHelper.success();
    }


    @GetMapping("/detail")
    @Operation(summary = "商品详情")
    public ResponseHelper<GoodsBean> detail(
            @Parameter(description = "id",required = true) @RequestParam String goodsId
    ) throws RemoteServiceException {
        GoodsBean detail = goodsLogic.detail(goodsId);
        return ResponseHelper.success(detail);
    }


    @GetMapping("/package/poplist")
    @Operation(summary = "套购商品-配置商品列表")
    public ResponseHelper<List<GoodsPackageBean>> popList(
            @Parameter(description = "商品id",required = true) @RequestParam String goodsId,
            @Parameter(description = "第几个商品框",required = true) @RequestParam Integer type
    ) throws RemoteServiceException {
        List<GoodsPackageBean> goodsPackageBeans = goodsLogic.popList(goodsId, type);
        return ResponseHelper.success(goodsPackageBeans);
    }

    @PostMapping("/add")
    @Operation(summary = "新增商品")
    public ResponseHelper add(
            @Parameter(description = "object",required = true) @RequestBody GoodsBean goodsBean,HttpServletRequest request)
            throws RemoteServiceException {
        goodsLogic.add(goodsBean,request);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @Operation(summary = "修改商品")
    public ResponseHelper update(
            @Parameter(description = "object",required = true) @RequestBody GoodsBean goodsBean,HttpServletRequest request)
            throws RemoteServiceException {
        goodsLogic.update(goodsBean,request);
        return ResponseHelper.success();
    }

    @PostMapping("/status/update")
    @Operation(summary = "商品上下架")
    public ResponseHelper change(
            @Parameter(description = "商品id",required = true) @RequestParam String goodsId,
            @Parameter(description = "商品状态:true上架 false下架",required = true) @RequestParam Boolean status
    ) throws RemoteServiceException {
        goodsLogic.updateStatus(goodsId, status);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @Operation(summary = "删除商品")
    public ResponseHelper delete(
            @Parameter(description = "商品id",required = true) @RequestParam String goodsId
    ) throws RemoteServiceException {
        goodsLogic.delete(goodsId);
        return ResponseHelper.success();
    }
    @PostMapping("/clone")
    @Operation(summary = "克隆商品")
    public ResponseHelper clone(
            @Parameter(description = "商品id",required = true) @RequestParam String goodsId
    ) throws RemoteServiceException {
        return ResponseHelper.success(goodsLogic.clone(goodsId));
    }


    @PostMapping("/goods/off/check")
    @Operation(summary = "下架商品前,检测是否秒杀商品")
    public ResponseHelper SecCheck(
            @Parameter(description = "商品id",required = true) @RequestParam String goodsId)
            throws RemoteServiceException {
        return ResponseHelper.success(goodsLogic.secCheck(goodsId));
    }

    @PostMapping("/goods/off/check/batch")
    @Operation(summary = "下架商品前,批量检测是否秒杀商品,返回秒杀的商品id列表")
    public ResponseHelper SecCheckBatch(
            @Parameter(description = "商品id",required = true) @RequestBody List<String> goodsIdList)
            throws RemoteServiceException {
        return ResponseHelper.success(goodsLogic.secCheckBatch(goodsIdList));
    }



    @PostMapping("/goods/on")
    @Operation(summary = "批量上架商品")
    public ResponseHelper goodsOn(
            @Parameter(description = "object",required = true) @RequestBody List<String> goodsIdList)
            throws RemoteServiceException {
        goodsLogic.goodsOnOff(goodsIdList,true);
        return ResponseHelper.success();
    }

    @PostMapping("/goods/off")
    @Operation(summary = "批量下架商品")
    public ResponseHelper goodsOf(
            @Parameter(description = "object",required = true) @RequestBody List<String> goodsIdList)
            throws RemoteServiceException {
        goodsLogic.goodsOnOff(goodsIdList,false);
        return ResponseHelper.success();
    }

    @PostMapping("/goods/template")
    @Operation(summary = "批量修改商品模板")
    public ResponseHelper goodsTemplate(
            @Parameter(description = "object",required = true) @RequestBody GoodsTemplateBean goodsTemplateBean,HttpServletRequest request)
            throws RemoteServiceException {
        goodsLogic.goodsTemplate(goodsTemplateBean,request);
        return ResponseHelper.success();
    }

    @PostMapping("/goods/batch/freight")
    @Operation(summary = "批量修改运费模板")
    public ResponseHelper goodsFreight(
            @Parameter(description = "object",required = true) @RequestBody GoodsFreightBean goodsFreightBean)
            throws RemoteServiceException {
        goodsLogic.goodsFreight(goodsFreightBean);
        return ResponseHelper.success();
    }


    @GetMapping("/package/user")
    @Operation(summary = "套购商品-指定人员列表")
    public ResponseHelper<List<User>> packageUserList(
            @Parameter(description = "商品id",required = true) @RequestParam(required = true) String goodsId,
            @Parameter(description = "商品id",required = false) @RequestParam(required = false) String mobile
    ) throws Exception {
        List<User> packageUseUserList = goodsLogic.getPackageUseUserList(goodsId, mobile);
        return ResponseHelper.success(packageUseUserList);
    }


    @PostMapping("/package/import")
    @Operation(summary = "套购商品-导入指定人员")
    public ResponseHelper<List<User>> packageUserList(
            MultipartFile multipartFile
    ) throws Exception {
        List<Object> objects = ExcelUtils.importExcel(multipartFile);
        List<User> users = goodsLogic.importPackageUser(objects);
        return ResponseHelper.success(users);
    }

    @Operation(summary = "复制分类商品信息")
    @GetMapping("/copy/categoryAndGoods")
    public ResponseHelper categoryAndGoods(
            @RequestParam String sourceCompanyId,
            @RequestParam String targetCompanyId,
            @RequestParam String targetCompanyName
    ){
        goodsLogic.categoryAndGoods(sourceCompanyId, targetCompanyId, targetCompanyName);
        return ResponseHelper.success();
    }

    @Operation(summary = "复制运费信息")
    @GetMapping("/copy/freightTemplate")
    public ResponseHelper copyFreightTemplate(
            @RequestParam String sourceCompanyId,
            @RequestParam String targetCompanyId,
            @RequestParam String targetCompanyName
    ){
        goodsLogic.copyFreightTemplate(sourceCompanyId, targetCompanyId, targetCompanyName);
        return ResponseHelper.success();
    }
}
