package com.gree.mall.contest.controller.pc.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.enums.OrderShareStatusEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderShareLogic;
import com.gree.mall.contest.plus.entity.OrderShare;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@RestController
@Tag(name = "订单结算管理", description = "订单结算管理")
@RequestMapping(value = "/order/share", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class OrderShareController {

    private final OrderShareLogic orderShareLogic;


    @GetMapping("/list")
    @Operation(summary = "列表")
    public ResponseHelper<Page<OrderShare>> list(
            HttpServletRequest request,
            @Parameter(description = "用户id", required = false) @RequestParam(required = false) String userId,
            @Parameter(description = "网点id", required = false) @RequestParam(required = false) String websitId,
            @Parameter(description = "分销结算id", required = false) @RequestParam(required = false) String orderShareId,
            @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
            @Parameter(description = "用户名", required = false) @RequestParam(required = false) String workUserName,
            @Parameter(description = "手机号", required = false) @RequestParam(required = false) String phone,
            @Parameter(description = "结算状态: ING=结算中  OVER=已结算  CANCEL=已取消 EXCEPTION=异常结算", required = false) @RequestParam(required = false) String status,
            @Parameter(description = "开始创建时间", required = false) @RequestParam(required = false) String startTime,
            @Parameter(description = "结束创建时间", required = false) @RequestParam(required = false) String endTime,
            @Parameter(description = "开始结算时间", required = false) @RequestParam(required = false) String startSettleTime,
            @Parameter(description = "结束结算时间", required = false) @RequestParam(required = false) String endSettleTime,
            @Parameter(description = "支付开始时间", required = false) @RequestParam(required = false) String startPayTime,
            @Parameter(description = "结束开始时间", required = false) @RequestParam(required = false) String endPayTime,
            @Parameter(description = "支付方式", required = false) @RequestParam(required = false) String payType,
            @Parameter(description = "页号", required = true) @RequestParam(required = true) Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam(required = true) Integer pageSize,
            @Parameter(description = "职位", required = false) @RequestParam(required = false) String position
    ) throws RemoteServiceException {
        IPage<OrderShare> list = orderShareLogic.list(request, userId, websitId, orderShareId, orderId, workUserName,
                phone, status, startTime, endTime, startSettleTime, endSettleTime, position, startPayTime, endPayTime,
                payType, pageNum, pageSize);
        return ResponseHelper.success(list);
    }

    @GetMapping("/confirm")
    @Operation(summary = "管理员手动确认分佣")
    public ResponseHelper confirm(@Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId) throws RemoteServiceException {
        orderShareLogic.confirm(orderId);
        return ResponseHelper.success();
    }


    @PostMapping("/remark")
    @Operation(summary = "备注")
    public ResponseHelper remark(
            @Parameter(description = "结算id", required = true) @RequestParam(required = true) String orderShareId,
            @Parameter(description = "备注内容", required = true) @RequestParam(required = true) String remark
    ) throws RemoteServiceException {
        orderShareLogic.remark(orderShareId, remark);
        return ResponseHelper.success();
    }


    @GetMapping("/export")
    @Operation(summary = "结算导出")
    public void export(HttpServletRequest request, HttpServletResponse response,
                       @Parameter(description = "用户id", required = false) @RequestParam(required = false) String userId,
                       @Parameter(description = "分销结算id", required = false) @RequestParam(required = false) String orderShareId,
                       @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
                       @Parameter(description = "网点id", required = false) @RequestParam(required = false) String websitId,
                       @Parameter(description = "用户名", required = false) @RequestParam(required = false) String workUserName,
                       @Parameter(description = "手机号", required = false) @RequestParam(required = false) String phone,
                       @Parameter(description = "结算状态: ING=结算中  OVER=已结算  CANCEL=已取消 EXCEPTION=异常结算", required = false) @RequestParam(required = false) String status,
                       @Parameter(description = "开始创建时间", required = false) @RequestParam(required = false) String startTime,
                       @Parameter(description = "结束创建时间", required = false) @RequestParam(required = false) String endTime,
                       @Parameter(description = "开始结算时间", required = false) @RequestParam(required = false) String startSettleTime,
                       @Parameter(description = "结束结算时间", required = false) @RequestParam(required = false) String endSettleTime,
                       @Parameter(description = "支付开始时间", required = false) @RequestParam(required = false) String startPayTime,
                       @Parameter(description = "结束开始时间", required = false) @RequestParam(required = false) String endPayTime,
                       @Parameter(description = "职位", required = false) @RequestParam(required = false) String position,
                       @Parameter(description = "支付方式", required = false) @RequestParam(required = false) String payType
    ) throws Exception {
        orderShareLogic.export2(request, response, userId, websitId, orderShareId, orderId, workUserName, phone, status,
                startTime, endTime, startSettleTime, endSettleTime, position, payType, startPayTime, endPayTime);
    }


    @ApiNotAuth
    @GetMapping("/worker/ordernum")
    @Operation(summary = "师傅本月的有效订单数量")
    public ResponseHelper<Long> workerOrderNum(
            @Parameter(description = "师傅手机号", required = true) @RequestParam(required = true) String mobile
    ) throws RemoteServiceException {
        Long shareNum = orderShareLogic.getShareNum(mobile);
        return ResponseHelper.success(shareNum);
    }

    @PostMapping("/status")
    @Operation(summary = "变更状态-线下结算-取消结算")
    public ResponseHelper status(
            @Parameter(description = "结算id", required = true) @RequestParam(required = true) String orderShareId,
            @Parameter(description = "OFFLINE=线下结算 CANCEL=取消结算", required = true) @RequestParam(required = true) OrderShareStatusEnum orderShareStatus
    ) {
        orderShareLogic.updateOrderStatus(orderShareId, orderShareStatus.toString());
        return ResponseHelper.success();
    }

    @PostMapping("/query/max/share/rate")
    @Operation(summary = "查询最大分账比例")
    public ResponseHelper<String> queryMaxShareRate(@Parameter(description = "商户id") @RequestParam(required = false) String id) throws Exception {
        String result = orderShareLogic.queryMaxShareRate(id);
        return ResponseHelper.success(result);
    }
}
