package com.gree.mall.contest.controller.pc.template;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.bean.template.TemplateBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.template.TemplateLogic;
import com.gree.mall.contest.plus.entity.CommonTemplate;
import com.gree.mall.contest.plus.entity.PubTemplate;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;

@Slf4j
@RestController
@Tag(name = "公共模板", description = "公共模板")
@RequestMapping(value = "/template", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class TemplateController {
    private final TemplateLogic templateLogic;

    @GetMapping("/list")
    @Operation(summary = "模板列表")
    public ResponseHelper<IPage<CommonTemplate>> list(
            @Parameter(description = "1=说明管理 2=水印管理",required = false) @RequestParam(required = false) Integer type,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize,
            HttpServletRequest request
    ) {
        IPage<CommonTemplate> list = templateLogic.list(type,pageNum, pageSize ,request);
        return ResponseHelper.success(list);
    }


    @PostMapping("/save")
    @Operation(summary = "创建模板")
    public ResponseHelper<String> save(
            @RequestBody TemplateBean templateBean,
            HttpServletRequest request
    ) throws ParseException {
        return ResponseHelper.success(templateLogic.save(templateBean,request));
    }


    @PostMapping("/update")
    @Operation(summary = "编辑模板")
    public ResponseHelper<IPage<CommonTemplate>> update(
            @RequestBody TemplateBean templateBean, HttpServletRequest request
    ) throws ParseException {
        return ResponseHelper.success(templateLogic.update(templateBean,request));
    }

    @PostMapping("/pub/update")
    @Operation(summary = "编辑公共模板")
    public ResponseHelper<String> updatePub(@RequestBody PubTemplate templateBean, HttpServletRequest request) {
        return ResponseHelper.success(templateLogic.updatePub(templateBean,request));
    }

    @GetMapping("/pub/query")
    @Operation(summary = "获取公共模板")
    public ResponseHelper<PubTemplate> queryPub(HttpServletRequest request) {
        return ResponseHelper.success(templateLogic.queryPub(request));
    }


    @PostMapping("/delete")
    @Operation(summary = "删除模板")
    public ResponseHelper<IPage<CommonTemplate>> delete(@Parameter(description = "模板id", required = true) @RequestParam String commonTemplateId) {
        return ResponseHelper.success(templateLogic.delete(commonTemplateId));
    }


    @GetMapping("/detail")
    @Operation(summary = "模板详情")
    public ResponseHelper<IPage<CommonTemplate>> detail(@Parameter(description = "模板id", required = true) @RequestParam String commonTemplateId) {
        return ResponseHelper.success(templateLogic.detail(commonTemplateId));
    }


}
