package com.gree.mall.contest.logic.common;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.commonmapper.AdminMapper;
import com.gree.mall.contest.enums.StateEnum;
import com.gree.mall.contest.enums.admin.RoleTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import com.gree.mall.contest.utils.ApplicationContextUtils;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.oss.OSSUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CommonLogic {

    @Value("${spring.profiles.active}")
    private String active;
    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    AdminCompanyWechatService AdminCompanyWechatService;
    @Autowired
    CommonFileService commonFileService;
    //    @Autowired
//    RegionService regionService;
    @Autowired
    AdminUserService adminUserService;
    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;
    @Autowired
    AdminMapper adminMapper;
    @Autowired
    AdminWebsitService adminWebsitService;
    @Autowired
    AdminWebsitRegionService adminWebsitRegionService;
    @Autowired
    UserService userService;
    @Autowired
    OSSUtil ossUtil;
    @Autowired
    SysConfigService sysConfigService;

    @Autowired
    LbsAmapService lbsAmapService;

    public Map<String, String> getOSSConfig() throws UnsupportedEncodingException {
        return ossUtil.getConfig();
    }

    public SysConfig getSysConfig() {
        SysConfig sysConfig = sysConfigService.lambdaQuery().last("limit 1").one();
        if (sysConfig == null) {
            throw new RemoteServiceException("系统缺少必要配置，请联系相关人员");
        }
        return sysConfig;
    }

    /**
     * 查询服务商的所有账号
     *
     * @param serviceProviderId
     * @return
     */
    public List<AdminUser> getAdminUserByServiceProviderId(String serviceProviderId) {
        List<AdminUser> list = adminUserService.lambdaQuery()
                .eq(AdminUser::getStatus, true)
                .eq(AdminUser::getServiceProviderId, serviceProviderId).list();
        return list;
    }


    /**
     * 获取用户信息
     *
     * @param mobile
     * @param flag
     * @return
     */
    public User getUserByMobile(String mobile, Integer flag) {
        if (StringUtils.isBlank(mobile) || flag == null) {
            throw new RemoteServiceException("获取用户信息失败，缺少必要的参数");
        }
        return userService.lambdaQuery()
                .eq(User::getMobile, mobile)
                .one();
    }

    /**
     * 查询当前PC用户
     *
     * @return
     */
    public AdminUserCom getAdminUser() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return this.getAdminUser(request);
    }

    /**
     * 获取帐号以及部门id集合
     */
    public AdminUserCom getAdminUser(HttpServletRequest request) {
        String userId = CommonUtils.getUserId(request);
        //方便测试用
        if (StringUtils.isBlank(userId) && !active.equals("prd")) {
            AdminUser admin = adminUserService.lambdaQuery().eq(AdminUser::getUserName, "admin").one();
            AdminUserCom adminUserCom = new AdminUserCom();
            BeanUtils.copyProperties(admin, adminUserCom);
            AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById("1");
            adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
            return adminUserCom;
        }
        AdminUser adminUser = adminUserService.getById(userId);
        if (adminUser == null) {
            return null;
        }
        if (!adminUser.getStatus()) {
            throw new RemoteServiceException("帐号已被禁用");
        }
        AdminUserCom adminUserCom = new AdminUserCom();
        BeanUtils.copyProperties(adminUser, adminUserCom);

        //所有帐号
        AdminUserCom account2 = this.websitAccount(adminUserCom, request);

        if (account2 != null) {
            return account2;
        }
        adminUserCom.setCompanyWechatIds(ListUtil.toList("1"));
        // TODO 写死后期处理
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById("1");
        adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
        adminUserCom.setLoginCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
        adminUserCom.setLoginCompanyName(adminCompanyWechat.getCompanyName());
        //暂不过滤权限
        return adminUserCom;
    }

    /**
     * 部门帐号
     */
    public AdminUserCom websitAccount(AdminUserCom adminUserCom, HttpServletRequest request) {
        AdminWebsit adminWebsit = adminWebsitService.getById(adminUserCom.getAdminWebsitId());
        adminUserCom.setAdminWebsit(adminWebsit);
        //非平台账号
        if (!adminUserCom.getUserName().equals("admin")) {
            List<AdminWebsit> list = new ArrayList<>();
            list.addAll(this.queryAllChild(list, adminUserCom.getAdminWebsitId()));
            List<String> websitIds = list.stream().map(AdminWebsit::getWebsitId).distinct().collect(Collectors.toList());
            websitIds.add(adminUserCom.getAdminWebsitId());
            adminUserCom.setAdminWebsitIds(websitIds.stream().distinct().collect(Collectors.toList()));

            //平台业务员的区id
            if (StringUtils.equals(adminUserCom.getType(), RoleTypeEnum.A.getCode())) {
                //区域id
                List<AdminWebsitRegion> adminWebsitRegions = adminWebsitRegionService.lambdaQuery().in(AdminWebsitRegion::getAdminWebsitId, adminUserCom.getAdminWebsitIds()).list();
                adminUserCom.setAreaIds(adminWebsitRegions.stream().map(AdminWebsitRegion::getAreaId).distinct().collect(Collectors.toList()));
            }

        }

        return adminUserCom;
    }

    public User getUser() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        String userId = CommonUtils.getUserId(request);
        User user = userService.getById(userId);
        return user;
    }

    /**
     * 递归查询所有数据
     */
    public List<AdminWebsit> queryAllChild(List<AdminWebsit> list, String id) {
        if (id == null || id.equals("0")) {
            return list;
        }
        List<AdminWebsit> adminWebsits = adminWebsitService.lambdaQuery().eq(AdminWebsit::getParentId, id).list();
        if (adminWebsits.size() == 0) {
            return list;
        }
        for (AdminWebsit adminWebsit : adminWebsits) {
            this.queryAllChild(list, adminWebsit.getWebsitId());
        }
        list.addAll(adminWebsits);
        return list;
    }

    /**
     * 上传附件
     *
     * @param file
     * @return
     * @throws IOException
     */
    public CommonFile uploadFile(MultipartFile file) throws IOException, RemoteServiceException {

        if (file == null || file.isEmpty()) {
            return null;
        }
        String uploadFilePath = file.getOriginalFilename().replaceAll("\\\\", "/");
        String uploadFileName = uploadFilePath.substring(
                uploadFilePath.lastIndexOf('/') + 1, uploadFilePath.lastIndexOf('.'));
        String uploadFileSuffix = uploadFilePath.substring(uploadFilePath.lastIndexOf('.') + 1, uploadFilePath.length());
        long currentTime = System.currentTimeMillis();
        BufferedOutputStream bufferedOutputStream = null;

        //限制大小为10m
        if (file.getSize() > 1024 * 1024 * 30) {
            throw new RemoteServiceException("文件大小超出限制");
        }

        try {
            String filePath = ossUtil.getFilePath() + "/" + currentTime + UUID.randomUUID() + "." + uploadFileSuffix;
            if (!ossUtil.uploadFile(filePath, file.getBytes())) {
                throw new RemoteServiceException("文件同步oss失败");
            }
            String ossUrl = ossUtil.getAccessUrl() + filePath;

            CommonFile commonFile = new CommonFile();
            commonFile.setUrl(ossUrl);
            commonFile.setName(uploadFileName);
            commonFile.setFileType(uploadFileSuffix);
            commonFile.setFileSize(BigDecimal.valueOf(file.getSize()));
            commonFile.setCreateTime(new Date());
            commonFileService.save(commonFile);
            return commonFile;
        } finally {
            if (bufferedOutputStream != null) {
                bufferedOutputStream.close();
            }
        }
    }


//    public List<Region> queryRegionList(List<String> pid){
//        return regionService.lambdaQuery().in(Region::getPid,pid).list();
//    }

    public String getFile(String key) throws IOException {
        String url = ossUtil.getUrlWw(key);
        return url;
    }

    public List<LbsAmap> city2(String city) {
        List<LbsAmap> citys = lbsAmapService.lambdaQuery()
                .eq(LbsAmap::getStatus, StateEnum.ON.getKey())
                .like(LbsAmap::getName, city)
                .eq(LbsAmap::getLevel, "city")
                .list();
        return citys;
    }


    public AdminCompanyWechat getAdminCompanyWechat(String subAppId) {
        AdminCompanyWechat one = adminCompanyWechatService.lambdaQuery().eq(AdminCompanyWechat::getSubAppId, subAppId).last("limit 1").one();
        return one;
    }

    /**
     * 附件归属绑定
     */
    public void bindFileFile(String objId, String objType, List<CommonFile> fileIds) {
        if (CollectionUtil.isEmpty(fileIds)) {
            return;
        }
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId, objId)
                .eq(CommonFile::getObjType, objType).remove();

        for (CommonFile file : fileIds) {
            file.setId(null);
            file.setObjId(objId);
            file.setObjType(objType);
        }
        commonFileService.saveBatch(fileIds);
    }

    /**
     * 查询附件列表
     */
    public List<CommonFile> queryFileByObjId(String objId, String objType) {
        return commonFileService.lambdaQuery()
                .eq(CommonFile::getObjId, objId)
                .eq(CommonFile::getObjType, objType)
                .orderByAsc(CommonFile::getCreateTime)
                .list();
    }

    /**
     * 查询附件列表
     */
    public List<String> queryFileUrlsByObjId(String objId, String objType) {
        List<CommonFile> commonFiles = this.queryFileByObjId(objId, objType);
        if (commonFiles.size() == 0) {
            return null;
        }
        return commonFiles.stream().map(CommonFile::getUrl).collect(Collectors.toList());
    }

    /**
     * 附件归属绑定
     */
    public void bindFile(String objId,String objType,List<String> fileIds){
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId,objId)
                .eq(CommonFile::getObjType,objType)
                .notIn(CommonFile::getId,fileIds).remove();

        for(int i =0;i< fileIds.size();i++) {
            //这里分开每次存储的时候，每个图片创建时间加1秒，方便图片排序用
            commonFileService.lambdaUpdate()
                    .set(CommonFile::getObjId, objId)
                    .set(CommonFile::getObjType, objType)
                    .set(CommonFile::getCreateTime, DateUtils.addSeconds(new Date(),i))
                    .eq(CommonFile::getId, fileIds.get(i))
                    .update();
        }
    }
}
