package com.gree.mall.contest.logic.coupon;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.ExcelData;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.coupon.*;
import com.gree.mall.contest.commonmapper.CouponDateMapper;
import com.gree.mall.contest.enums.UserTypeEnum;
import com.gree.mall.contest.enums.coupon.CouponActiveTypeEnum;
import com.gree.mall.contest.enums.coupon.CouponCrowdEnum;
import com.gree.mall.contest.enums.coupon.CouponFlagEnum;
import com.gree.mall.contest.enums.coupon.CouponTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import com.gree.mall.contest.utils.excel.ExcelUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class CouponLogic {

    private final CouponService couponService;
    private final CouponGoodsService couponGoodsService;
    private final CouponDateMapper couponDateMapper;
    private final UserCouponService userCouponService;
    private final CouponUserService couponUserService;
    private final UserService userService;
    private final CommonLogic commonLogic;

    /**
     * 优惠券列表
     */
    public IPage<CouponPageBean> page(String couponName, String couponType, String flag, Integer pageNum, Integer pageSize, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return couponDateMapper.pageCoupon(new Page<>(pageNum, pageSize), companyWechatIds, couponName, couponType, flag);
    }


    public void export(HttpServletRequest request, HttpServletResponse response, String couponName, String couponType, String flag) throws Exception {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        List<CouponExportBean> couponExportBeans = couponDateMapper.exportCoupon(companyWechatIds, couponName, couponType, flag);
        //List<CouponExportBean> couponSelfBeans =  couponDateMapper.exportSelfCoupon(companyWechatIds, couponName, couponType, flag);
//        if(CollectionUtils.isNotEmpty(couponSelfBeans)){
//            couponExportBeans.addAll(couponSelfBeans);
//        }

        String[] titles = {"优惠券名称", "发放数量", "领取数量", "使用数量", "优惠券金额", "领取来源-网点名称", "领取来源-业务员昵称", "领取来源-业务员电话", "优惠券状态", "领券客户昵称", "领券客户电话",
                "用券时间", "订单号", "订单实付金额"};
        ExcelData excelData = new ExcelData();
        List<List<Object>> rows = new ArrayList<>();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (CouponExportBean couponExportBean : couponExportBeans) {
            List<Object> row = new ArrayList<>();
            row.add(couponExportBean.getCouponName());
            row.add(couponExportBean.getCouponAmount());
            row.add(couponExportBean.getReceiveAmount());
            row.add(couponExportBean.getUsedAmount());
            row.add(couponExportBean.getCouponValue());
            row.add(couponExportBean.getWebsitName());
            row.add(couponExportBean.getWorkName());
            row.add(couponExportBean.getWorkPhone());
            row.add(couponExportBean.getStatus() ? "已使用" : "未使用");
            row.add(couponExportBean.getNickName());
            row.add(couponExportBean.getMobile());
            row.add(couponExportBean.getUsedTime() == null ? "" : simpleDateFormat.format(couponExportBean.getUsedTime()));
            row.add(couponExportBean.getOrderId());
            row.add(couponExportBean.getPayAmount() == null ? "" : couponExportBean.getPayAmount().toString());
            rows.add(row);
        }
        String fileName = String.format("优惠券导出_%s.xlsx", simpleDateFormat.format(new Date()));
        excelData.setTitles(Arrays.asList(titles));
        excelData.setName("优惠券");
        excelData.setRows(rows);
        ExcelUtils.exportExcel(request, response, fileName, excelData, -1, null);
    }

    /**
     * 券详情
     *
     * @param couponId
     * @return
     */
    public CouponDetailBean detail(String couponId) {
        Coupon coupon = couponService.getById(couponId);
        CouponDetailBean couponBean = new CouponDetailBean();
        BeanUtils.copyProperties(coupon, couponBean);
        couponBean.setGoodsList(couponGoodsService.lambdaQuery().eq(CouponGoods::getCouponId, couponId).list());
        return couponBean;
    }

    /**
     * 新增券
     *
     * @param couponBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(HttpServletRequest request, CouponBean couponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        if (couponBean.getActiveType() == 1) {
            if (Objects.isNull(couponBean.getActiveStartTime()) || Objects.isNull(couponBean.getActiveEndTime())) {
                throw new RemoteServiceException("使用时间区间不能为空！");
            }
            if (couponBean.getActiveEndTime().compareTo(couponBean.getActiveStartTime()) < 0) {
                throw new RemoteServiceException("使用结束时间不能少于开始时间！");
            }
        }
        couponBean.setFlag(CouponFlagEnum.WAIT.toString());
        couponBean.setLeftAmount(couponBean.getCouponAmount());
        couponBean.setCreateTime(new Date());
        couponBean.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        couponBean.setCompanyName(adminUser.getLoginCompanyName());
        couponBean.setActiveType(couponBean.getActiveDay() != null ? 2 : 1);
        couponService.save(couponBean);
        String couponId = couponBean.getCouponId();
        if (couponBean.getCouponType().trim().equals(CouponTypeEnum.GOODS.getName()) && couponBean.getGoodsList() != null) {
            List<CouponGoods> couponGoodsList = new ArrayList<>();
            for (CouponGoods couponGoods : couponBean.getGoodsList()) {
                couponGoods.setCouponId(couponId);
                couponGoods.setCreateTime(new Date());
                couponGoods.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponGoods.setCompanyName(adminUser.getLoginCompanyName());
                couponGoodsList.add(couponGoods);
            }
            couponGoodsService.saveBatch(couponGoodsList);
        }

        //指定会员
        if (couponBean.getReceiveCrowd().equals(CouponCrowdEnum.SPECIFY.getFlag()) && couponBean.getUserList() != null) {
            List<CouponUser> couponUserList = new ArrayList<>();
//            List<UserCoupon> userCouponList = new ArrayList<>();
            for (CouponUserTypeBean couponUserTypeBean : couponBean.getUserList()) {
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(couponUserTypeBean.getUserId());
                couponUser.setUserName(couponUserTypeBean.getUserName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUserList.add(couponUser);
//                if (couponUserTypeBean.getType().equals(UserTypeEnum.SERVICE.toString())) {
//                    userCouponList.add(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
//                }

            }

            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }

//            if (userCouponList.size() > 0) {
//                userCouponService.saveBatch(userCouponList);
//            }

        }
        //全部业务员
        if (couponBean.getReceiveCrowd() == CouponCrowdEnum.ALL_SERVICE.getFlag()) {
            List<UserCoupon> userCouponList = new ArrayList<>();
            List<User> userList = userService.lambdaQuery()
                    .eq(User::getType, UserTypeEnum.SERVICE.toString())
                    .eq(User::getCompanyWechatId, adminUser.getLoginCompanyWechatId())
                    .list();
            List<CouponUser> couponUserList = new ArrayList<>();
            for (User user : userList) {
                userCouponList.add(assignCoupon(adminUser, couponBean, user.getUserId()));
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(user.getUserId());
                couponUser.setUserName(user.getNickName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUserList.add(couponUser);
            }
            if (userCouponList.size() > 0) {
                userCouponService.saveBatch(userCouponList);
            }
            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }
        }
        //全部普通用户
        if (couponBean.getReceiveCrowd().equals(CouponCrowdEnum.ALL_GENRERAL.getFlag())) {
            List<CouponUser> couponUserList = new ArrayList<>();
            List<User> userList = userService.lambdaQuery()
                    .eq(User::getType, UserTypeEnum.GENERAL.toString())
                    .eq(User::getCompanyWechatId, adminUser.getLoginCompanyWechatId())
                    .list();
            for (User user : userList) {
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(user.getUserId());
                couponUser.setUserName(user.getNickName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUserList.add(couponUser);
            }
            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }
        }


    }


    private UserCoupon assignCoupon(AdminUserCom adminUserCom, CouponBean couponBean, String userId) {
        UserCoupon userCoupon = new UserCoupon();
        userCoupon.setCompanyWechatId(adminUserCom.getLoginCompanyWechatId());
        userCoupon.setCouponName(adminUserCom.getLoginCompanyName());
        userCoupon.setCouponId(couponBean.getCouponId());
        userCoupon.setUserId(userId);
        userCoupon.setReceiveTime(new Date());
        userCoupon.setCouponName(couponBean.getCouponName());
        userCoupon.setCouponType(couponBean.getCouponType());
        userCoupon.setOrderAmount(couponBean.getOrderAmount());
        userCoupon.setCouponValue(couponBean.getCouponValue());
        userCoupon.setStatus(false);
        //userCoupon.setTransferType(true);
        userCoupon.setLeftShareTimes(couponBean.getShareTimes());
        if (couponBean.getActiveDay() == null) {
            couponBean.setActiveDay(0);
        }
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.COMMON.getActiveType())) {
            userCoupon.setActiveStartTime(couponBean.getActiveStartTime());
            userCoupon.setActiveEndTime(couponBean.getActiveEndTime());
        }
        //当日起多少天
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.TODAY.getActiveType())) {
            userCoupon.setActiveStartTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.date()) + " 00:00:00"));
            userCoupon.setActiveEndTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), couponBean.getActiveDay())) + " 23:59:59"));
        }
        //次日起多少天
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.NEXTDAY.getActiveType())) {
            userCoupon.setActiveStartTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), 1)) + " 00:00:00"));
            userCoupon.setActiveEndTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), couponBean.getActiveDay() + 1)) + " 23:59:59"));
        }
        userCoupon.setState(new Date().before(couponBean.getActiveEndTime()) && new Date().after(couponBean.getActiveStartTime()));
        return userCoupon;
    }

    public void reissueCoupon(HttpServletRequest request, ReissueCouponBean reissueCouponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        String loginCompanyWechatId = adminUser.getLoginCompanyWechatId();
        String loginCompanyName = adminUser.getLoginCompanyName();

        Coupon coupon = couponService.getById(reissueCouponBean.getCouponId());
        if (coupon == null) {
            throw new RemoteServiceException("优惠券不存在");
        }
        if (coupon.getTransferType() != null && !coupon.getTransferType()) {
            throw new RemoteServiceException("优惠券不可以补发");
        }
        if (coupon.getObtainEndTime().compareTo(new Date()) <= 0) {
            throw new RemoteServiceException("优惠券超过领取时间");
        }
        List<CouponUserTypeBean> userIds = reissueCouponBean.getUserIds();
        if (CollectionUtil.isEmpty(userIds)) {
            userIds = couponDateMapper.reissueCouponUser(loginCompanyWechatId, coupon.getCouponId());
        }
        List<CouponUser> couponUserList = new ArrayList<>();
        List<UserCoupon> userCouponList = new ArrayList<>();
        for (CouponUserTypeBean couponUserTypeBean : userIds) {
            CouponUser couponUser = new CouponUser();
            couponUser.setCouponId(coupon.getCouponId());
            couponUser.setUserId(couponUserTypeBean.getUserId());
            couponUser.setUserName(couponUserTypeBean.getUserName());
            couponUser.setCreateTime(new Date());
            couponUser.setCompanyWechatId(loginCompanyWechatId);
            couponUser.setCompanyName(loginCompanyName);
            couponUserList.add(couponUser);
            CouponBean couponBean = new CouponBean();
            BeanUtils.copyProperties(coupon, couponBean);
            userCouponList.add(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
        }

        if (couponUserList.size() > 0) {
            couponUserService.saveBatch(couponUserList);
        }

        if (userCouponList.size() > 0) {
            userCouponService.saveBatch(userCouponList);
        }

    }


    /**
     * 修改券
     *
     * @param couponBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(HttpServletRequest request, CouponBean couponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        String loginCompanyName = adminUser.getLoginCompanyName();
        String loginCompanyWechatId = adminUser.getLoginCompanyWechatId();

        Coupon coupon = couponService.getById(couponBean.getCouponId());
        couponBean.setCreateTime(null);
        Date now = new Date();
        if (coupon.getFlag().trim().equals(CouponFlagEnum.WAIT.toString())) {
            //待发布处理
            couponService.updateById(couponBean);
            //清除商品
            if (couponBean.getGoodsList() != null && couponBean.getGoodsList().size() > 0) {
                couponGoodsService.lambdaUpdate()
                        .eq(CouponGoods::getCouponId, couponBean.getCouponId()).remove();
                for (CouponGoods couponGoods : couponBean.getGoodsList()) {
                    couponGoods.setCouponId(couponBean.getCouponId());
                    couponGoods.setCreateTime(now);
                    couponGoods.setCompanyWechatId(loginCompanyWechatId);
                    couponGoods.setCompanyName(loginCompanyName);
                    couponGoodsService.save(couponGoods);
                }
            }

            if (couponBean.getUserList() != null && couponBean.getUserList().size() > 0) {
                couponUserService.lambdaUpdate()
                        .eq(CouponUser::getCouponId, couponBean.getCouponId())
                        .remove();
                List<CouponUser> couponUserList = new ArrayList<>();
                for (CouponUserTypeBean couponUserTypeBean : couponBean.getUserList()) {
                    CouponUser couponUser = new CouponUser();
                    couponUser.setCouponId(couponBean.getCouponId());
                    couponUser.setUserId(couponUserTypeBean.getUserId());
                    couponUser.setUserName(couponUserTypeBean.getUserName());
                    couponUser.setCreateTime(new Date());
                    couponUser.setCompanyWechatId(loginCompanyWechatId);
                    couponUser.setCompanyName(loginCompanyName);
                    couponUserList.add(couponUser);
                    if (couponUserTypeBean.getType().equals(UserTypeEnum.SERVICE.toString())) {
                        UserCoupon userCoupon = userCouponService.lambdaQuery()
                                .eq(UserCoupon::getCouponId, couponBean.getCouponId())
                                .eq(UserCoupon::getUserId, couponUserTypeBean.getUserId())
                                //.eq(UserCoupon::getTransferType, true)
                                .one();
                        if (userCoupon == null) {
                            userCouponService.save(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
                        }

                    }
                }
                if (couponUserList.size() > 0) {
                    couponUserService.saveBatch(couponUserList);
                }
            }
        } else if (!coupon.getFlag().trim().equals(CouponFlagEnum.CANCEL.toString()) && coupon.getDisplayTime().before(now) && coupon.getObtainEndTime().after(now)) {
            //进行中处理
            if (coupon.getCouponAmount() > couponBean.getCouponAmount()) {
                throw new RemoteServiceException("修改优惠券发放总量时只能增加不能减少！");
            }
            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponBean.getCouponId())
                    .set(StringUtils.isNotBlank(couponBean.getCouponName()), Coupon::getCouponName, couponBean.getCouponName())
                    .set(couponBean.getCouponAmount() > coupon.getCouponAmount(), Coupon::getCouponAmount, couponBean.getCouponAmount())
                    .set(!Objects.isNull(couponBean.getSortNum()), Coupon::getSortNum, couponBean.getSortNum())
                    .set(StringUtils.isNotBlank(couponBean.getNote()), Coupon::getNote, couponBean.getNote())
                    .set(StringUtils.isNotBlank(couponBean.getRemark()), Coupon::getRemark, couponBean.getRemark())
                    .update();
            //更改已领取的券名称
            if (StringUtils.isNotBlank(couponBean.getCouponName())) {
                userCouponService.lambdaUpdate()
                        .eq(UserCoupon::getCouponId, couponBean.getCouponId())
                        .set(UserCoupon::getCouponName, couponBean.getCouponName())
                        .update();
            }
        }
    }

    /**
     * 券标志
     *
     * @param couponId
     * @param startDate
     * @param endDate
     * @param flag
     */
    public void updateFlag(String couponId, String startDate, String endDate, String flag) throws RemoteServiceException {
        if (flag.trim().equals(CouponFlagEnum.START.toString())) {
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
                throw new RemoteServiceException("时间不能为空！");
            }
            if (endDate.compareTo(startDate) < 0) {
                throw new RemoteServiceException("结束时间不能少于开始时间！");
            }

            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponId)
                    .set(Coupon::getReleaseStartTime, startDate)
                    .set(Coupon::getReleaseEndTime, endDate)
                    .set(Coupon::getFlag, CouponFlagEnum.START.toString())
                    .update();
        } else {
            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponId)
                    .set(Coupon::getFlag, CouponFlagEnum.CANCEL.toString())
                    .update();

            userCouponService.lambdaUpdate()
                    .set(UserCoupon::getState, false)
                    .set(UserCoupon::getLeftShareTimes, 0)
                    .set(UserCoupon::getActiveEndTime, DateUtil.formatDate(DateUtil.date()) + " 00:00:00")
                    .eq(UserCoupon::getCouponId, couponId)
                    .eq(UserCoupon::getStatus, false)
                    .update();
        }
    }

    /**
     * 券数据
     *
     * @param couponId
     * @return
     */
    public CouponDataBean data(String couponId) {
        return couponDateMapper.data(couponId);
    }


    /**
     * 修改优惠券是否可以转让
     */
    public void updateUserCouponTransfer(List<String> userIds, Boolean transferType) {
        userCouponService.lambdaUpdate()
                .set(UserCoupon::getTransferType, transferType)
                .in(UserCoupon::getUserId, userIds)
                .update();
    }

    /**
     * 我的优惠券
     */
    public IPage<UserCoupon> userCouponList(String userId, Integer pageNo, Integer pageSize) {
        return userCouponService.lambdaQuery()
                .eq(UserCoupon::getUserId, userId)
                .orderByDesc(UserCoupon::getReceiveTime)
                .page(new Page<>(pageNo, pageSize));
    }

    /**
     * 领取优惠券明细
     *
     * @param userCouponId
     * @param pageNo
     * @param pageSize
     * @return
     */
    public IPage<UserCouponBean> receiveCouponDetail(String userCouponId, Integer pageNo, Integer pageSize) {

        IPage page = userCouponService.lambdaQuery()
                .eq(UserCoupon::getTransferor, userCouponId)
                .eq(UserCoupon::getTransferType, 0)
                .orderByDesc(UserCoupon::getReceiveTime)
                .page(new Page<>(pageNo, pageSize));

        List<UserCouponBean> list = new ArrayList<>();
        for (Object o : page.getRecords()) {
            UserCoupon userCoupon = (UserCoupon) o;
            UserCouponBean couponUser = new UserCouponBean();
            BeanUtils.copyProperties(userCoupon, couponUser);
            User user = userService.getById(userCoupon.getUserId());
            couponUser.setUserName(user.getNickName());
            couponUser.setMobile(user.getMobile());
            list.add(couponUser);
        }
        page.setRecords(list);
        return page;

    }

    public IPage<User> userList(HttpServletRequest request,
                                String tel, Date registerTime, String keyword, String type, Integer pageNo, Integer pageSize) {

        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return userService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), User::getCompanyWechatId, companyWechatIds)
                .eq(StringUtils.isNotEmpty(type), User::getType, type)
                .ge(registerTime != null, User::getCreateTime, registerTime)
                .like(StringUtils.isNotEmpty(tel), User::getMobile, tel.trim())
                .like(StringUtils.isNotEmpty(keyword), User::getNickName, keyword.trim())
                .orderByAsc(User::getCreateTime)
                .page(new Page<>(pageNo, pageSize));
    }

    /**
     * 订单取消,恢复优惠券
     */
    @Transactional
    public void refundCoupon(String userCouponId) {
        if (StringUtils.isEmpty(userCouponId)) {
            return;
        }
        userCouponService.lambdaUpdate()
                .set(UserCoupon::getStatus, false)
                .set(UserCoupon::getUsedTime, null)
                .eq(UserCoupon::getId, userCouponId)
                .update();

        if (StringUtils.isEmpty(userCouponId)) {
            return;
        }
        log.info("订单取消,恢复用户优惠券id{}", userCouponId);
        UserCoupon userCoupon = userCouponService.getById(userCouponId);
        if (userCoupon.getTransferType() != null && userCoupon.getTransferType()) {
            userCoupon.setLeftShareTimes(userCoupon.getLeftShareTimes() + 1);
            userCoupon.updateById();
        }
    }


    /**
     * 导入楼小贤优惠券
     */
//    @Transactional
//    public void importLxx(MultipartFile file) throws IOException, ParseException {
//        List<Object> datas = ExcelUtils.importExcel(file);
//        int i = 1;
//
//        List<Coupon> couponList = new ArrayList<>();
//        List<CouponUser> couponUserList = new ArrayList<>();
//        List<UserCoupon> userCouponList = new ArrayList<>();
//        for (Object o : datas) {
//            String companyWechatId = "1";
//            String companyName = "嘉讯茂商城";
//            i++;
//            List<Object> row = (List<Object>) o;
//            String couponName = (String) row.get(0);
//            String mobile = (String) row.get(1);
//            BigDecimal amount = new BigDecimal(row.get(2).toString());
//            Date startTime = DateUtils.parseDate(row.get(3).toString());
//            Date endTime = DateUtils.parseDate(row.get(4).toString());
//            if (StringUtils.isEmpty(couponName) || StringUtils.isEmpty(mobile) || amount == null || startTime == null || endTime == null) {
//                throw new RemoteServiceException("请检查第" + i + "行数据是否完整");
//            }
//            User user = userService.lambdaQuery().eq(User::getMobile, mobile).eq(User::getCompanyWechatId, companyWechatId).one();
//            if (user == null)
//                throw new RemoteServiceException("手机号为" + mobile + "的找不到对应用户");
//
//            //生成优惠券
//            String couponId = "C" + com.baomidou.mybatisplus.core.toolkit.IdWorker.getIdStr();
//            Coupon coupon = new Coupon();
//            coupon.setCouponId(couponId);
//            coupon.setCouponName(couponName);
//            coupon.setCouponType(CouponTypeEnum.SATISFY.toString());
//            coupon.setActiveStartTime(startTime);
//            coupon.setActiveEndTime(endTime);
//            coupon.setReceiveCrowd(2);
//            coupon.setCouponValue(amount);
//            coupon.setObtainStartTime(startTime);
//            coupon.setObtainEndTime(endTime);
//            coupon.setDisplayTime(startTime);
//            coupon.setFlag("WAIT");
//            coupon.setRemark(mobile);
//            coupon.setShareTimes(1);
//            coupon.setCompanyWechatId(companyWechatId);
//            coupon.setCompanyName(companyName);
//            couponList.add(coupon);
//
//            CouponUser couponUser = new CouponUser();
//            couponUser.setCouponId(couponId);
//            couponUser.setUserId(user.getUserId());
//            couponUser.setUserName(user.getNickName());
//            couponUser.setCreateTime(new Date());
//            couponUser.setCompanyWechatId(companyWechatId);
//            couponUser.setCompanyName(companyName);
//            couponUserList.add(couponUser);
//
//            //发到师傅手上的券
//            UserCoupon userCoupon = new UserCoupon();
//            userCoupon.setUserId(user.getUserId());
//            userCoupon.setCouponId(couponId);
//            userCoupon.setCouponName(couponName);
//            userCoupon.setCouponValue(coupon.getCouponValue());
//            userCoupon.setCouponType(coupon.getCouponType());
//            userCoupon.setStatus(false);
//            userCoupon.setState(true);
//            userCoupon.setReceiveTime(new Date());
//            userCoupon.setActiveStartTime(startTime);
//            userCoupon.setActiveEndTime(endTime);
//            userCoupon.setCompanyWechatId(companyWechatId);
//            userCoupon.setCompanyName(companyName);
//            userCoupon.setTransferType(false);
//            userCoupon.setLeftShareTimes(1);
//            userCouponList.add(userCoupon);
//        }
//        //新增券
//        couponService.saveBatch(couponList);
//        //新增券和人对应关系
//        couponUserService.saveBatch(couponUserList);
//        //发券
//        userCouponService.saveBatch(userCouponList);
//    }

}
