package com.gree.mall.contest.utils;

import cn.hutool.core.date.DateUtil;
import com.gree.mall.contest.exception.RemoteServiceException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;

public class CommonUtils {

    public static List<String> initList(List<String> list,int size){
        int listSize = list.size();
        if(listSize < size){
            for(int i = listSize;i < size;i++){
                list.add("");
            }
        }
        return list;
    }

    public static List<Object> initList2(List<Object> list,int size){
        int listSize = list.size();
        if(listSize < size){
            for(int i = listSize;i < size;i++){
                list.add("");
            }
        }
        return list;
    }

    /**
     * pc来源的接口环境
     * @return
     */
    public static Boolean isPc(){
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        String appid = request.getHeader("APPID");
        return StringUtils.isBlank(appid);
    }


    public static String getAPPID(){
        HttpServletRequest request = RequestContextHolder.getRequestAttributes() != null ? ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest() : null;
        String appid = request.getHeader("APPID");
        return appid;
    }

    public static String getUserId() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getUserId(request);
    }

    public static String getUserId(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("token");
        }
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JwtUtils.parseToken(token).getUserId();
        } catch (Exception e) {
            return null;
        }
    }

    public static String getUsername() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getUsername(request);
    }

    public static String getUsername(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JwtUtils.parseToken(token).getUsername();
        } catch (Exception e) {
            return null;
        }
    }

    public static String getRealName() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getUsername(request);
    }

    public static String getRealName(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JwtUtils.parseToken(token).getRealName();
        } catch (Exception e) {
            return null;
        }
    }

    public static String getUserType() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getUserType(request);
    }

    public static String getUserType(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JwtUtils.parseToken(token).getUserType();
        } catch (Exception e) {
            return null;
        }
    }

    public static boolean isExpiration() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return isExpiration(request);
    }

    public static boolean isExpiration(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            return true;
        }
        final Date expiration = JwtUtils.parseToken(token).getExpiration();
        if (expiration.before(DateUtil.date())) {
            return true;
        }
        return false;
    }

    public static String getIssuer() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getIssuer(request);
    }

    public static String getIssuer(HttpServletRequest request) {
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JwtUtils.parseToken(token).getIssuer();
        } catch (Exception e) {
            return null;
        }
    }

    public static String getToken() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return getToken(request);
    }

    /**
     * 获取token
     */
    public static String getToken(HttpServletRequest request) {
        String token = request.getHeader("x-token");
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("token");
        }
        return token;
    }


    public static void downloadFile(String fileName, HttpServletResponse response) throws IOException {
        InputStream inputStream = CommonUtils.class.getResourceAsStream("/template/" + fileName);
        response.setContentType("Content-Type:application/octet-stream");
        response.setHeader("Content-disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));
        OutputStream outputStream = response.getOutputStream();
        byte[] buffer = new byte[1024];
        int len;
        while ((len = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, len);
        }
        inputStream.close();
        outputStream.close();
    }


    /**
     * 组装服务单修改记录的日志
     *
     * @return
     */
    public static String supplyLog(String title, Object oldValue, Object newValue) {
        if (oldValue instanceof String) {
            oldValue = oldValue == null ? "" : oldValue;
            newValue = newValue == null ? "" : newValue;
            if (!StringUtils.equals((String) oldValue, (String) newValue)) {
                return "【" + title + "】修改前：" + oldValue + ",修改后：" + newValue + "。";
            }
        }
        if (oldValue instanceof Integer) {
            oldValue = oldValue == null ? 0 : oldValue;
            newValue = newValue == null ? 0 : newValue;
            if (!((Integer) oldValue).equals((Integer) newValue)) {
                return "【" + title + "】修改前：" + oldValue + ",修改后：" + newValue + "。";
            }
        }
        if(oldValue instanceof Date){
            long o = oldValue == null ? 0 : ((Date) oldValue).getTime();
            long n = newValue == null ? 0 : ((Date) newValue).getTime();
            if(o != n){
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                return "【" + title + "】修改前：" + sdf.format(oldValue) + ",修改后：" + sdf.format(newValue) + "。";
            }
        }
        if(oldValue == null && newValue != null){
            oldValue = oldValue == null ? "" : oldValue;
            newValue = newValue == null ? "" : newValue;
            return "【" + title + "】修改前：" + oldValue + ",修改后：" + newValue + "。";

        }
        if(oldValue != null && newValue == null){
            oldValue = oldValue == null ? "" : oldValue;
            newValue = newValue == null ? "" : newValue;
            return "【" + title + "】修改前：" + oldValue + ",修改后：" + newValue + "。";
        }
        return "";
    }





    /**
     * 根据URL地址获取文件
     * @param path URL网络地址
     * @return File
     */
    public static File getFileByHttpURL(String path, String address){
        String newUrl = path.split("[?]")[0];
        String[] suffix = newUrl.split("/");
        //得到最后一个分隔符后的名字
        String fileName = suffix[suffix.length - 1];
        File file = null;
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try{
            file = new File(address+"/"+fileName);//创建临时文件
            URL urlFile = new URL(path);
            inputStream = urlFile.openStream();
            outputStream = new FileOutputStream(file);

            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead=inputStream.read(buffer,0,8192))!=-1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        }catch (Exception e) {
            throw new RemoteServiceException(e.getMessage());
        }finally {
            try {
                if (null != outputStream) {
                    outputStream.close();
                }
                if (null != inputStream) {
                    inputStream.close();
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return file;
    }



    /**
     * 计算两个时间段的工作时长
     * @param startTime 工作开始时间
     * @param endTime 工作结束时间
     * @param taskStartTime 任务开始时间
     * @param taskEndTime 任务结束时间
     * @return
     */
    public static long calculateValidWorkTime(LocalTime startTime, LocalTime endTime, LocalTime taskStartTime, LocalTime taskEndTime) {
        long totalMills = 0;

        // 任务开始是否在工作时间内
        if (taskStartTime.isAfter(startTime) && taskStartTime.isBefore(endTime)) {
            // 任务结束是否在工作时间内
            if (taskEndTime.isAfter(startTime) && taskEndTime.isBefore(endTime)) {
                // 任务全部在有效工作时间内
                totalMills = ChronoUnit.MILLIS.between(taskStartTime, taskEndTime);
            } else {
                // 任务从工作时间开始到结束不完全在有效工作时间内
                totalMills = ChronoUnit.MILLIS.between(taskStartTime, endTime);
            }
        } else {
            // 任务开始时间超出工作时间
            LocalTime nextWorkStart = startTime.isAfter(taskStartTime) ? startTime : taskStartTime;
            LocalTime nextWorkEnd = endTime.isAfter(taskEndTime) ? taskEndTime : endTime;
            totalMills = ChronoUnit.MILLIS.between(nextWorkStart, nextWorkEnd);
        }

        return totalMills;
    }

}
