package com.gree.mall.contest.utils;

import com.gree.mall.contest.constant.Constant;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.io.Encoders;
import io.jsonwebtoken.security.Keys;
import io.jsonwebtoken.security.SignatureException;

import javax.crypto.SecretKey;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


public class JwtUtils {

//    private static final String SECRET = "zfire@2025_base_code_security_key_123456"; // 64位HMAC-SHA512密钥
//    private static final long EXPIRATION = 7200L; // 默认有效期2小时（秒）
//    private static final String ISSUER = "base-code-system"; // 签发者
    // HMAC-SHA512签名算法
//    private static final SecretKey SECRET_KEY = Keys.hmacShaKeyFor(SECRET.getBytes());

    private static final String secret = "uH31bHvL+P1jl5OvHfqLPhJDkfqHWIEL6e7HEpTMLozNAlGH88SUlXlWc7KK/SgWh1oZjvW7f8d43Bm0+J7hnw==";
    private static final Long expiration = 7200L;
    private static final SecretKey secretKey = Keys.hmacShaKeyFor(secret.getBytes());

    // 1. 生成安全的随机密钥（自动满足长度要求）
    public static void generateAndSaveKey() {
        SecretKey key = Jwts.SIG.HS512.key().build(); // 自动生成 512-bit 密钥
        String base64Key = Encoders.BASE64.encode(key.getEncoded());
        System.out.println("安全密钥 (Base64): " + base64Key);
        // 将 base64Key 存储到安全配置中（如环境变量、配置中心）
    }

    /**
     * 生成JWT令牌
     * @param userId    用户ID
     * @param username  用户名
     * @param realName  真实姓名
     * @param userType 用户类型
     * @param issuer 用户角色
     * @return JWT令牌
     */
    public static String generateToken(String userId, String username, String realName, String userType, String issuer) {
        Map<String, Object> claims = new HashMap<>(3);
        claims.put(Constant.CLAIM_USER_ID, userId);
        claims.put(Constant.CLAIM_USERNAME, username);
        claims.put(Constant.CLAIM_REAL_NAME, realName);
        claims.put(Constant.CLAIM_USER_TYPE, userType);

        return Jwts.builder()
                .issuer(issuer)
                .subject(username)
                .claims(claims)
                .issuedAt(new Date())
                .expiration(new Date(System.currentTimeMillis() + expiration * 1000))
                .signWith(secretKey, Jwts.SIG.HS512)
                .compact();
    }

    /**
     * 解析JWT令牌
     * @param token JWT令牌
     * @return 解析结果包装对象
     */
    public static JwtParseResult parseToken(String token) {
        try {
            Claims claims = Jwts.parser()
                    .verifyWith(secretKey)
                    .build()
                    .parseSignedClaims(token)
                    .getPayload();

            return JwtParseResult.success()
                    .userId(claims.get(Constant.CLAIM_USER_ID, String.class))
                    .username(claims.getSubject())
                    .realName(claims.get(Constant.CLAIM_REAL_NAME, String.class))
                    .userType(claims.get(Constant.CLAIM_USER_TYPE, String.class))
                    .expiration(claims.getExpiration())
                    .issuer(claims.getIssuer());
        } catch (ExpiredJwtException e) {
            return JwtParseResult.fail(1001, "令牌已过期");
        } catch (SignatureException e) {
            return JwtParseResult.fail(1002, "无效签名");
        } catch (Exception e) {
            return JwtParseResult.fail(1003, "令牌解析失败");
        }
    }



    /**
     * 刷新令牌
     * @param token 原令牌
     * @return 新令牌（仅当原令牌有效且未过期时刷新）
     */
    public static String refreshToken(String token, String issuer) {
        JwtParseResult result = parseToken(token);
        if (!result.isSuccess()) {
            return null;
        }
        return generateToken(
                result.getUserId(),
                result.getUsername(),
                result.getRealName(),
                result.getUserType(),
                issuer
        );
    }

    /**
     * JWT解析结果包装类
     */
    public static class JwtParseResult {
        private boolean success;
        private int code;
        private String message;
        private String userId;
        private String username;
        private String realName;
        private String userType;
        private Date expiration;
        private String issuer;

        public static JwtParseResult success() {
            JwtParseResult result = new JwtParseResult();
            result.success = true;
            result.code = 200;
            result.message = "success";
            return result;
        }

        public static JwtParseResult fail(int code, String message) {
            JwtParseResult result = new JwtParseResult();
            result.success = false;
            result.code = code;
            result.message = message;
            return result;
        }

        // 链式调用设置字段
        public JwtParseResult userId(String userId) {
            this.userId = userId;
            return this;
        }

        public JwtParseResult username(String username) {
            this.username = username;
            return this;
        }

        public JwtParseResult realName(String realName) {
            this.realName = realName;
            return this;
        }

        public JwtParseResult userType(String userType) {
            this.userType = userType;
            return this;
        }

        public JwtParseResult expiration(Date expiration) {
            this.expiration = expiration;
            return this;
        }

        public JwtParseResult issuer(String issuer) {
            this.issuer = issuer;
            return this;
        }

        // Getter 方法
        public boolean isSuccess() { return success; }
        public int getCode() { return code; }
        public String getMessage() { return message; }
        public String getUserId() { return userId; }
        public String getUsername() { return username; }
        public String getRealName() { return realName; }
        public String getUserType() { return userType; }
        public Date getExpiration() { return expiration; }
        public String getIssuer() { return issuer; }
    }
}
